﻿using System;
using System.Windows.Forms;
using Microsoft.SqlServer.Management.Smo;
using System.Text;
using System.Collections.Generic;
using System.IO;
using System.Data.SqlClient;
using System.Data;

namespace SQLComparator
{
    public partial class MainFrm : Form
    {
        private bool masterDatabaseReady;
        private bool slaveDatabaseReady;
        private int errorCount = 0;
        private StringBuilder databaseScript = new StringBuilder();
        private StringBuilder tableScript = new StringBuilder();
        private StringBuilder columnScript = new StringBuilder();
        private StringBuilder columnScriptMH = new StringBuilder();
        private StringBuilder columnScriptSH = new StringBuilder();
        private StringBuilder columnScriptM = new StringBuilder();
        private StringBuilder columnScriptS = new StringBuilder();
        private List<string> lackTable = new List<string>();
        private List<string> lackView = new List<string>();
        private List<string> badView = new List<string>();
        private List<string> lackProcedue = new List<string>();
        private List<string> badProcedue = new List<string>();
        private string[] reason = new string[5];
        private bool tableWrong;
        private bool columnWrong;
        private string reportLocation;
        private string revertReportLocation;
        private StringBuilder timemessage = new StringBuilder();
        [System.Runtime.InteropServices.DllImport("kernel32.dll")]
        static extern uint GetTickCount();     

        public MainFrm()
        {
            InitializeComponent();
        }

        /// <summary>
        /// 读取master 信息
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void BtnMasterTest_Click(object sender, EventArgs e)
        {
            RtxtResult.Text = string.Empty;
            masterDatabaseReady = false;
            TreeMaster.Nodes.Clear();
            TreeMaster.BeginUpdate();
            string server = TxtMasterServer.Text;
            string userName = TxtMasterUserName.Text;
            string password = TxtMasterPassword.Text;
            string database = TxtMasterDatabase.Text;

            if (!ConnectServer(TreeMaster, server, userName, password, database))
            {
                TreeMaster.EndUpdate();
                LblStatus.Text = "连接失败...";
                MessageBox.Show("主数据库服务器连接失败", "提示", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            TreeMaster.EndUpdate();
            masterDatabaseReady = true;
        }

        /// <summary>
        /// 读取slave 信息
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void BtnSlaveTest_Click(object sender, EventArgs e)
        {
            RtxtResult.Text = string.Empty;
            slaveDatabaseReady = false;
            TreeSlave.Nodes.Clear();
            TreeSlave.BeginUpdate();
            string server = TxtSlaveServer.Text;
            string userName = TxtSlaveUserName.Text;
            string password = TxtSlavePassword.Text;
            string database = TxtSlaveDatabase.Text;

            if (!ConnectServer(TreeSlave, server, userName, password, database))
            {
                TreeSlave.EndUpdate();
                LblStatus.Text = "连接失败...";
                MessageBox.Show("附数据库服务器连接失败", "提示", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            TreeSlave.EndUpdate();
            slaveDatabaseReady = true;
        }

        private bool ConnectServer(TreeView tree, string serverName, string userName, string password, string databaseName)
        {
            LblStatus.Text = "连接...";
            if (DatabaseInfo.TestConnection(serverName, userName, password))
            {
                Server server = DatabaseInfo.GetServer(serverName, userName, password);
                //server.GetSmoObject(
                Database database = server.Databases[databaseName];
                if (database == null) return false;

                TreeNode dataBaseNode = new TreeNode();
                dataBaseNode.Text = database.Name;
                dataBaseNode.Tag = database;

                DatabaseView(database, dataBaseNode, serverName, userName, password, databaseName);

                tree.Nodes.Add(dataBaseNode);
                return true;
            }
            else
            {
                return false;
            }
        }

        /// <summary>
        /// 列出数据库信息
        /// </summary>
        /// <param name="treeView"></param>
        private void DatabaseView(Database database, TreeNode databaseNode, string severname, string username, string password, string databasename)
        {
            List<string>[] TVP = TableViewProcecure(severname, username, password, databasename);
            DateTime dt = DateTime.Now;
            LblStatus.Text = "加载表...";

            bool start = false;
            // 加载表
            TreeNode tableNodes = new TreeNode("表");
            if (TVP[0] != null)
            {
                for (int i = 0; i < TVP[0].Count; i++)
                {
                    Table table = database.Tables[TVP[0][i]];
                    TreeNode tableNode = new TreeNode(table.Name);
                    GetColumn(table.Columns, tableNode);
                    GetCheck(table.Checks, tableNode);
                    GetTrigger(table.Triggers, tableNode);
                    GetIndex(table.Indexes, tableNode);
                    tableNodes.Nodes.Add(tableNode);
                }
                DateTime dtt = DateTime.Now;
                timemessage.Append(",加载表"+ Convert.ToString(( dtt - dt).TotalSeconds)+"秒");




                start = false;
                LblStatus.Text = "加载视图...";
                // 视图
                TreeNode viewNodes = new TreeNode("视图");
                for (int i = 0; i < TVP[1].Count; i++)
                {
                    Microsoft.SqlServer.Management.Smo.View view = database.Views[TVP[1][i]];
                    Application.DoEvents();

                    TreeNode viewNode = new TreeNode(view.Name);
                    viewNode.Tag = view;
                    GetColumn(view.Columns, viewNode);
                    viewNodes.Nodes.Add(viewNode);
                }
                DateTime dttt = DateTime.Now;
                timemessage.Append(",加载视图" + Convert.ToString((dttt - dtt).TotalSeconds)+"秒");


                // 存储过程
                Application.DoEvents();
                LblStatus.Text = "加载存储过程...";
                TreeNode storedProcedureNodes = new TreeNode("可编程性");
                start = false;
                for (int i = 0; i < TVP[2].Count; i++)
                {
                    Application.DoEvents();
                    StoredProcedure storedProcedure = database.StoredProcedures[TVP[2][i]];

                    TreeNode columnNode = new TreeNode(storedProcedure.Name);
                    columnNode.Tag = storedProcedure;
                    storedProcedureNodes.Nodes.Add(columnNode);
                }
                DateTime dtttt = DateTime.Now;
                timemessage.Append(",加载存储过程" + Convert.ToString((dtttt - dttt).TotalSeconds)+"秒");


                databaseNode.Nodes.Add(tableNodes);
                databaseNode.Nodes.Add(viewNodes);
                databaseNode.Nodes.Add(storedProcedureNodes);

                LblStatus.Text = "加载完成" + timemessage.ToString();
                timemessage = new StringBuilder();
                

            }
        }

        /// <summary>
        /// 加载表列
        /// </summary>
        /// <param name="table"></param>
        /// <param name="tableNode"></param>
        private void GetColumn(ColumnCollection columns, TreeNode tableNode)
        {
            TreeNode columnNodes = new TreeNode("列");
            foreach (Column column in columns)
            {
                Application.DoEvents();
                string columnName;
                if (column.InPrimaryKey)
                    columnName = string.Format("PK, {0}, {1}", column.DataType.Name, column.Nullable ? "null" : "not null");
                else
                    columnName = string.Format("{0}, {1}", column.DataType.Name, column.Nullable ? "null" : "not null");

                TreeNode columnNode = new TreeNode(columnName);
                columnNode.Name = column.Name;
                columnNode.Tag = column;

                columnNodes.Nodes.Add(columnNode);
            }

            tableNode.Nodes.Add(columnNodes);
        }

        /// <summary>
        /// 加载表索引
        /// </summary>
        /// <param name="table"></param>
        /// <param name="tableNode"></param>
        private void GetIndex(IndexCollection indexs, TreeNode tableNode)
        {
            TreeNode indexNodes = new TreeNode("索引");
            foreach (Index index in indexs)
            {
                Application.DoEvents();
                TreeNode columnNode = new TreeNode(string.Format("{0} {1}", index.Name, GetIndexDescription(index)));
                columnNode.Tag = index;
                indexNodes.Nodes.Add(columnNode);
            }

            tableNode.Nodes.Add(indexNodes);
        }

        /// <summary>
        /// 获得索引的描述
        /// </summary>
        /// <param name="index">索引</param>
        /// <returns>获得索引的描述</returns>
        private string GetIndexDescription(Microsoft.SqlServer.Management.Smo.Index index)
        {
            StringBuilder builder = new StringBuilder();
            builder.Append("(");
            if (index.IsUnique)
            {
                builder.Append("唯一，");
            }
            else
            {
                builder.Append("不唯一，");
            }
            if (index.IsClustered)
            {
                builder.Append("聚集)");
            }
            else
            {
                builder.Append("非聚集)");
            }
            return builder.ToString();
        }

        /// <summary>
        /// 加载表索引
        /// </summary>
        /// <param name="table"></param>
        /// <param name="tableNode"></param>
        private void GetTrigger(TriggerCollection triggers, TreeNode tableNode)
        {
            TreeNode triggeNodes = new TreeNode("触发器");
            foreach (Trigger trigger in triggers)
            {
                Application.DoEvents();
                TreeNode columnNode = new TreeNode(trigger.Name);
                columnNode.Tag = trigger;
                triggeNodes.Nodes.Add(columnNode);
            }

            tableNode.Nodes.Add(triggeNodes);
        }

        /// <summary>
        /// 加载表约束
        /// </summary>
        /// <param name="table"></param>
        /// <param name="tableNode"></param>
        private void GetCheck(CheckCollection checks, TreeNode tableNode)
        {
            TreeNode triggeNodes = new TreeNode("约束");
            foreach (Check check in checks)
            {
                Application.DoEvents();
                TreeNode columnNode = new TreeNode(check.Name);
                columnNode.Tag = check;
                triggeNodes.Nodes.Add(columnNode);
            }

            tableNode.Nodes.Add(triggeNodes);
        }

        /// <summary>
        /// 加载表约束
        /// </summary>
        /// <param name="table"></param>
        /// <param name="tableNode"></param>
        private void GetStoredProcedures(StoredProcedureCollection storedProcedures, TreeNode tableNode)
        {
            bool start = false;

            TreeNode storedProcedureNodes = new TreeNode("存储过程");
            for (int i = 0; i < storedProcedures.Count; i++)
            {
                Application.DoEvents();
                StoredProcedure storedProcedure = storedProcedures[i];
                if (start && storedProcedure.IsSystemObject) break;
                if (!start && !storedProcedure.IsSystemObject) start = true;
                if (storedProcedure.IsSystemObject) continue;

                TreeNode columnNode = new TreeNode(storedProcedure.Name);
                columnNode.Tag = storedProcedure;
                storedProcedureNodes.Nodes.Add(columnNode);
            }

            tableNode.Nodes.Add(storedProcedureNodes);
        }

        /// <summary>
        /// 比较
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void BtnComprare_Click(object sender, EventArgs e)
        {
            errorCount = 0;
            RtxtResult.Text = string.Empty;
            if (!masterDatabaseReady || !slaveDatabaseReady)
            {
                MessageBox.Show("请选择比较数据库", "提示", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            #region 正比
            LblStatus.Text = "开始比较...";

            TreeNode masterMain = TreeMaster.Nodes[0];
            TreeNode tableMasterNode = masterMain.Nodes[0];
            TreeNode viewMasterNode = masterMain.Nodes[1];
            TreeNode storeProcedureMasterNode = masterMain.Nodes[2];

            TreeNode slaveMain = TreeSlave.Nodes[0];
            TreeNode tableSlaveNode = slaveMain.Nodes[0];
            TreeNode viewSlaveNode = slaveMain.Nodes[1];
            TreeNode storeProcedureSlaveNode = slaveMain.Nodes[2];

            LblStatus.Text = "开始比较表...";
            // 比较表
            CompareTable(tableMasterNode, tableSlaveNode);

            LblStatus.Text = "开始比较视图...";
            // 比较视图
            CompareView(viewMasterNode, viewSlaveNode);

            LblStatus.Text = "开始比较存储过程...";
            // 比较存储过程
            CompareStoreProcedure(storeProcedureMasterNode, storeProcedureSlaveNode);
            LblStatus.Text = "比较完成";
            if (errorCount > 0)
            {
                LblStatus.Text += string.Format(" ({0}错误)", errorCount);
            }
            AddNote("完成");

            if (string.IsNullOrEmpty(reportLocation))
                reportLocation = Path.GetTempFileName() + ".htm";

            WrongPage();
            #endregion

            #region 反比
            string oldReportLocation = reportLocation;
            if (string.IsNullOrEmpty(revertReportLocation))
            {
                revertReportLocation = Path.GetTempFileName() + ".htm";
            }

            reportLocation = revertReportLocation;

            errorCount = 0;
            LblStatus.Text = "开始反向比较...";

            masterMain = TreeSlave.Nodes[0];
            tableMasterNode = masterMain.Nodes[0];
            viewMasterNode = masterMain.Nodes[1];
            storeProcedureMasterNode = masterMain.Nodes[2];

            slaveMain = TreeMaster.Nodes[0];
            tableSlaveNode = slaveMain.Nodes[0];
            viewSlaveNode = slaveMain.Nodes[1];
            storeProcedureSlaveNode = slaveMain.Nodes[2];

            LblStatus.Text = "开始反向比较表...";
            // 比较表
            CompareTable(tableMasterNode, tableSlaveNode);

            LblStatus.Text = "开始反向比较视图...";
            // 比较视图
            CompareView(viewMasterNode, viewSlaveNode);

            LblStatus.Text = "开始反向比较存储过程...";
            // 比较存储过程
            CompareStoreProcedure(storeProcedureMasterNode, storeProcedureSlaveNode);
            LblStatus.Text = "反向比较完成";
            if (errorCount > 0)
            {
                LblStatus.Text += string.Format(" ({0}错误)", errorCount);
            }
            AddNote("完成");
            WrongPage();
            reportLocation = oldReportLocation;
            #endregion
        }

        private void CompareTable(TreeNode tableMasterNode, TreeNode tableSlaveNode)
        {
            foreach (TreeNode master in tableMasterNode.Nodes)
            {
                Application.DoEvents();
                bool bfind = false;
                foreach (TreeNode slave in tableSlaveNode.Nodes)
                {
                    if (master.Text.ToLower() == slave.Text.ToLower())
                    {
                        bfind = true;

                        // 列
                        CompareColumn(master.Text, master.Nodes[0], slave.Nodes[0]);
                        // 约束
                        CompareCheck(master.Text, master.Nodes[1], slave.Nodes[1]);
                        // 触发
                        CompareTrigger(master.Text, master.Nodes[2], slave.Nodes[2]);
                        // 索引
                        CompareIndex(master.Text, master.Nodes[3], slave.Nodes[3]);
                        if (tableWrong)
                        {
                            tableWrong = false;
                            AddHeadString(master.Text);
                            tableScript.Append(columnScript.ToString());
                            tableScript.Append("</table><div id=\"TOPEnd\" class=\"TopClass\"><a href=\"#TOP\">TOP</a></div><br/>");
                            columnScript = new StringBuilder();
                            columnScriptMH = new StringBuilder();
                            columnScriptSH = new StringBuilder();
                            columnScriptM = new StringBuilder();
                            columnScriptS = new StringBuilder();
                            columnWrong = false;
                        }
                    }
                }

                if (!bfind)
                {
                    AddNote("缺少表：" + master.Text);
                    lackTable.Add(master.Text);
                    if (reason[0] == null)
                        reason[0] = "缺少表：";
                }
            }
        }
        private void CompareColumn(string table, TreeNode tableMasterNode, TreeNode tableSlaveNode)
        {
            foreach (TreeNode master in tableMasterNode.Nodes)
            {
                columnScriptM = new StringBuilder();
                columnScriptS = new StringBuilder();
                columnScriptMH = new StringBuilder();
                columnScriptSH = new StringBuilder();
                columnWrong = false;
                Application.DoEvents();
                bool bfind = false;
                foreach (TreeNode slave in tableSlaveNode.Nodes)
                {
                    if (master.Name.ToLower() == slave.Name.ToLower())
                    {
                        bfind = true;
                        // 列
                        Column m = master.Tag as Column;
                        Column s = slave.Tag as Column;

                        if (master.Text.ToLower() != slave.Text.ToLower())
                        {
                            AddNote(string.Format("{0}列{1}字段类型不同", table, master.Name));
                            tableWrong = true;
                            columnWrong = true;
                            columnScriptM.Append("<td class=\"td-03\">");
                            columnScriptM.Append(master.Text);
                            columnScriptM.Append("</td>");
                            columnScriptS.Append("<td class=\"td-03\">");
                            columnScriptS.Append(slave.Text);
                            columnScriptS.Append("</td>");
                        }
                        else
                        {
                            columnScriptM.Append("<td class=\"td-01\">");
                            columnScriptM.Append("</td>");
                            columnScriptS.Append("<td class=\"td-02\">");
                            columnScriptS.Append("</td>");
                        }

                        if (m.DataType.MaximumLength != s.DataType.MaximumLength)
                        {
                            AddNote(string.Format("{0}列{1}字段长度不同", table, master.Name));
                            tableWrong = true;
                            columnWrong = true;
                            columnScriptM.Append("<td class=\"td-03\">");
                            columnScriptM.Append(m.DataType.MaximumLength);
                            columnScriptM.Append("</td>");
                            columnScriptS.Append("<td class=\"td-03\">");
                            columnScriptS.Append(s.DataType.MaximumLength);
                            columnScriptS.Append("</td>");
                        }
                        else
                        {
                            columnScriptM.Append("<td class=\"td-01\">");
                            columnScriptM.Append("</td>");
                            columnScriptS.Append("<td class=\"td-02\">");
                            columnScriptS.Append("</td>");
                        }

                        if (m.DataType.NumericPrecision != s.DataType.NumericPrecision)
                        {
                            AddNote(string.Format("{0}列{1}字段精度不同", table, master.Name));
                            tableWrong = true;
                            columnWrong = true;
                            columnScriptM.Append("<td class=\"td-03\">");
                            columnScriptM.Append(m.DataType.NumericPrecision.ToString());
                            columnScriptM.Append("</td>");
                            columnScriptS.Append("<td class=\"td-03\">");
                            columnScriptS.Append(s.DataType.NumericPrecision.ToString());
                            columnScriptS.Append("</td>");
                        }
                        else
                        {
                            columnScriptM.Append("<td class=\"td-01\">");
                            columnScriptM.Append("</td>");
                            columnScriptS.Append("<td class=\"td-02\">");
                            columnScriptS.Append("</td>");
                        }

                        if (m.DataType.NumericScale != s.DataType.NumericScale)
                        {
                            AddNote(string.Format("{0}列{1}字段小数位不同", table, master.Name));
                            tableWrong = true;
                            columnWrong = true;
                            columnScriptM.Append("<td class=\"td-03\">");
                            columnScriptM.Append(m.DataType.NumericScale);
                            columnScriptM.Append("</td>");
                            columnScriptS.Append("<td class=\"td-03\">");
                            columnScriptS.Append(s.DataType.NumericScale);
                            columnScriptS.Append("</td>");
                        }
                        else
                        {
                            columnScriptM.Append("<td class=\"td-01\">");
                            columnScriptM.Append("</td>");
                            columnScriptS.Append("<td class=\"td-02\">");
                            columnScriptS.Append("</td>");
                        }

                        if (m.InPrimaryKey != s.InPrimaryKey)
                        {
                            AddNote(string.Format("{0}列{1}主键不同", table, master.Name));
                            tableWrong = true;
                            columnWrong = true;
                            columnScriptM.Append("<td class=\"td-03\">");
                            columnScriptM.Append(m.InPrimaryKey.ToString());
                            columnScriptM.Append("</td>");
                            columnScriptS.Append("<td class=\"td-03\">");
                            columnScriptS.Append(s.InPrimaryKey.ToString());
                            columnScriptS.Append("</td>");
                        }
                        else
                        {
                            columnScriptM.Append("<td class=\"td-01\">");
                            columnScriptM.Append("</td>");
                            columnScriptS.Append("<td class=\"td-02\">");
                            columnScriptS.Append("</td>");
                        }

                        if (m.IsForeignKey != s.IsForeignKey)
                        {
                            AddNote(string.Format("{0}列{1}外键不同", table, master.Name));
                            tableWrong = true;
                            columnWrong = true;
                            columnScriptM.Append("<td class=\"td-03\">");
                            columnScriptM.Append(m.IsForeignKey.ToString());
                            columnScriptM.Append("</td>");
                            columnScriptS.Append("<td class=\"td-03\">");
                            columnScriptS.Append(s.IsForeignKey.ToString());
                            columnScriptS.Append("</td>");
                        }
                        else
                        {
                            columnScriptM.Append("<td class=\"td-01\">");
                            columnScriptM.Append("</td>");
                            columnScriptS.Append("<td class=\"td-02\">");
                            columnScriptS.Append("</td>");
                        }
                        if (m.Identity != s.Identity)
                        {
                            AddNote(string.Format("{0}列{1}自增不同", table, master.Name));
                            tableWrong = true;
                            columnWrong = true;
                            columnScriptM.Append("<td class=\"td-03\">");
                            columnScriptM.Append(m.Identity.ToString());
                            columnScriptM.Append("</td>");
                            columnScriptS.Append("<td class=\"td-03\">");
                            columnScriptS.Append(s.Identity.ToString());
                            columnScriptS.Append("</td>");
                        }
                        else
                        {
                            columnScriptM.Append("<td class=\"td-01\">");
                            columnScriptM.Append("</td>");
                            columnScriptS.Append("<td class=\"td-02\">");
                            columnScriptS.Append("</td>");
                        }
                        if (m.IdentityIncrement != s.IdentityIncrement)
                        {
                            AddNote(string.Format("{0}列{1}自增长度不同", table, master.Name));
                            tableWrong = true;
                            columnWrong = true;
                            columnScriptM.Append("<td class=\"td-03\">");
                            columnScriptM.Append(m.IdentityIncrement.ToString());
                            columnScriptM.Append("</td>");
                            columnScriptS.Append("<td class=\"td-03\">");
                            columnScriptS.Append(s.IdentityIncrement.ToString());
                            columnScriptS.Append("</td>");
                        }
                        else
                        {
                            columnScriptM.Append("<td class=\"td-01\">");
                            columnScriptM.Append("</td>");
                            columnScriptS.Append("<td class=\"td-02\">");
                            columnScriptS.Append("</td>");
                        }
                        if (m.Nullable != s.Nullable)
                        {
                            AddNote(string.Format("{0}列{1}Nullable不同", table, master.Name));
                            tableWrong = true;
                            columnWrong = true;
                            columnScriptM.Append("<td class=\"td-03\">");
                            columnScriptM.Append(m.Nullable.ToString());
                            columnScriptM.Append("</td>");
                            columnScriptS.Append("<td class=\"td-03\">");
                            columnScriptS.Append(s.Nullable.ToString());
                            columnScriptS.Append("</td>");
                        }
                        else
                        {
                            columnScriptM.Append("<td class=\"td-01\">");
                            columnScriptM.Append("</td>");
                            columnScriptS.Append("<td class=\"td-02\">");
                            columnScriptS.Append("</td>");
                        }
                        if (columnWrong)
                        {
                            AddColumnHeadString(master.Name);
                            columnScriptMH.Append(columnScriptM.ToString());
                            columnScriptSH.Append(columnScriptS.ToString());
                            columnScript.Append(columnScriptMH.ToString());
                            columnScript.Append(columnScriptSH.ToString());
                        }
                        break;
                    }
                }
                if (!bfind)
                {
                    AddNote(table + "缺少列：" + master.Name);
                    PublicPartAboutTable(master.Name, "缺少列");
                }
            }
        }
        private void CompareCheck(string table, TreeNode tableMasterNode, TreeNode tableSlaveNode)
        {
            foreach (TreeNode master in tableMasterNode.Nodes)
            {
                Application.DoEvents();
                bool bfind = false;
                foreach (TreeNode slave in tableSlaveNode.Nodes)
                {
                    if (master.Text.ToLower() == slave.Text.ToLower())
                    {
                        bfind = true;
                        // 列
                        Check m = master.Tag as Check;
                        Check s = slave.Tag as Check;

                        if (m.Text.Trim(' ', '\r', '\n') != s.Text.Trim(' ', '\r', '\n'))
                        {
                            AddNote(string.Format("{0}约束{1}不同", table, master.Text));
                            PublicPartAboutTable(master.Text, "约束不同");
                        }
                        break;
                    }
                }

                if (!bfind)
                {
                    AddNote(table + "缺少约束：" + master.Text);
                    PublicPartAboutTable(master.Text, "缺少约束");
                }
            }
        }
        private void CompareTrigger(string table, TreeNode tableMasterNode, TreeNode tableSlaveNode)
        {
            foreach (TreeNode master in tableMasterNode.Nodes)
            {
                Application.DoEvents();
                bool bfind = false;
                foreach (TreeNode slave in tableSlaveNode.Nodes)
                {
                    if (master.Text.ToLower() == slave.Text.ToLower())
                    {
                        bfind = true;
                        // 列
                        Trigger m = master.Tag as Trigger;
                        Trigger s = slave.Tag as Trigger;

                        if (m.TextBody.Trim(' ', '\r', '\n') != s.TextBody.Trim(' ', '\r', '\n'))
                        {
                            AddNote(string.Format("{0}触发{1}不同", table, master.Text));
                            PublicPartAboutTable(master.Text, "触发不同");
                        }
                        break;
                    }
                }

                if (!bfind)
                {
                    AddNote(table + "缺少触发：" + master.Text);
                    PublicPartAboutTable(master.Text, "缺少触发");
                }
            }
        }
        private void CompareIndex(string table, TreeNode tableMasterNode, TreeNode tableSlaveNode)
        {
            foreach (TreeNode master in tableMasterNode.Nodes)
            {
                Application.DoEvents();
                bool bfind = false;
                foreach (TreeNode slave in tableSlaveNode.Nodes)
                {
                    if (master.Text.ToLower() == slave.Text.ToLower())
                    {
                        bfind = true;
                        // 列
                        Index m = master.Tag as Index;
                        Index s = slave.Tag as Index;

                        System.Collections.Specialized.StringCollection msc = m.Script();
                        System.Collections.Specialized.StringCollection ssc = s.Script();

                        if (msc.Count != ssc.Count)
                        {
                            AddNote(string.Format("{0}索引{1}不同", table, master.Text));
                            PublicPartAboutTable(master.Text, "索引不同");
                        }
                        if (msc[0] != ssc[0])
                        {
                            AddNote(string.Format("{0}索引{1}不同", table, master.Text));
                            PublicPartAboutTable(master.Text, "索引不同");
                        }
                        break;
                    }
                }

                if (!bfind)
                {
                    AddNote(table + "缺少索引：" + master.Text);
                    PublicPartAboutTable(master.Text, "缺少索引");
                }
            }
        }
        private void CompareView(TreeNode tableMasterNode, TreeNode tableSlaveNode)
        {
            foreach (TreeNode master in tableMasterNode.Nodes)
            {
                Application.DoEvents();
                bool bfind = false;
                foreach (TreeNode slave in tableSlaveNode.Nodes)
                {
                    if (master.Text.ToLower() == slave.Text.ToLower())
                    {
                        bfind = true;
                        Microsoft.SqlServer.Management.Smo.View m = master.Tag as Microsoft.SqlServer.Management.Smo.View;
                        Microsoft.SqlServer.Management.Smo.View s = slave.Tag as Microsoft.SqlServer.Management.Smo.View;

                        if (m.TextBody.Trim(' ', '\r', '\n') != s.TextBody.Trim(' ', '\r', '\n'))
                        {
                            AddNote(string.Format("视图{0}内容不一致", master.Text));
                            badView.Add(master.Text);
                            if (reason[1] == null)
                                reason[1] = "视图内容不一致：";
                        }
                        break;
                    }
                }

                if (!bfind)
                {
                    AddNote("缺少视图：" + master.Text);
                    lackView.Add(master.Text);
                    if (reason[2] == null)
                        reason[2] = "缺少视图：";
                }
            }
        }
        private void CompareStoreProcedure(TreeNode tableMasterNode, TreeNode tableSlaveNode)
        {
            foreach (TreeNode master in tableMasterNode.Nodes)
            {
                Application.DoEvents();
                bool bfind = false;
                foreach (TreeNode slave in tableSlaveNode.Nodes)
                {
                    if (master.Text.ToLower() == slave.Text.ToLower())
                    {
                        bfind = true;
                        StoredProcedure m = master.Tag as StoredProcedure;
                        StoredProcedure s = slave.Tag as StoredProcedure;

                        if (m.TextBody.Trim(' ', '\r', '\n') != s.TextBody.Trim(' ', '\r', '\n'))
                        {
                            AddNote(string.Format("存储过程{0}内容不一致", master.Text));
                            badProcedue.Add(master.Text);
                            if (reason[3] == null)
                                reason[3] = "存储过程内容不一致：";
                        }

                        break;
                    }
                }

                if (!bfind)
                {
                    AddNote("缺少存储过程：" + master.Text);
                    lackProcedue.Add(master.Text);
                    if (reason[4] == null)
                        reason[4] = "缺少存储过程：";
                }
            }
        }

        private void AddNote(string message)
        {
            Application.DoEvents();
            errorCount++;
            if (message != "完成")
                RtxtResult.Text += errorCount.ToString() + ". ";
            RtxtResult.Text += message + "\r\n";
        }

        public void WrongPage()
        {
            string wrongpage = tableScript.ToString() + PublicPartAboutDataBase(reason, lackTable, badView, lackView, badProcedue, lackProcedue);
            wrongpage = SQLComparator.Properties.Resources.DiffPage.Replace("$abc$", wrongpage);

            File.WriteAllText(reportLocation, wrongpage, Encoding.UTF8);
            System.Diagnostics.Process.Start(reportLocation);
            databaseScript = new StringBuilder();
            tableScript = new StringBuilder();
            columnScript = new StringBuilder();
            columnScriptMH = new StringBuilder();
            columnScriptSH = new StringBuilder();
            columnScriptM = new StringBuilder();
            columnScriptS = new StringBuilder();
            lackTable = new List<string>();
            lackView = new List<string>();
            badView = new List<string>();
            lackProcedue = new List<string>();
            badProcedue = new List<string>();
            tableWrong = false;
            columnWrong = false;
            reason = new string[5];
        }

        /// <summary>
        /// 添加行的开始，即列名(公共的)
        /// </summary>
        /// <param name="columnHead"></param>
        public void AddColumnHeadString(string columnHead)
        {
            columnScriptMH.Append("<tr>");
            columnScriptMH.Append("<td class=\"td-01\">");
            columnScriptMH.Append(columnHead);
            columnScriptMH.Append("</td>");
            columnScriptSH.Append("<tr>");
            columnScriptSH.Append("<td class=\"td-02\">");
            columnScriptSH.Append(columnHead);
            columnScriptMH.Append("</td>");
        }

        /// <summary>
        /// 添加表的头
        /// </summary>
        /// <param name="tablename"></param>
        public void AddHeadString(string tablename)
        {
            tableScript.Append("<table class=\"Table-00\" cellspacing=\"1\" cellpadding=\"0\">");
            tableScript.Append("<tr>");
            tableScript.Append("<td class=\"td-00\" colspan=\"10\">");
            tableScript.Append(tablename + "</td>");
            tableScript.Append("</tr>");

            tableScript.Append("<tbody id=\"tdnt_admingroups\">");
            tableScript.Append("<tr>");
            tableScript.Append("<td class=\"td-00\">");
            tableScript.Append("代码");
            tableScript.Append("</td>");
            tableScript.Append("<td class=\"td-00\">");
            tableScript.Append("类型");
            tableScript.Append("</td>");
            tableScript.Append("<td class=\"td-00\">");
            tableScript.Append("     长度");
            tableScript.Append("</td>");
            tableScript.Append("<td class=\"td-00\">");
            tableScript.Append("     精度");
            tableScript.Append("</td>");
            tableScript.Append("  <td class=\"td-00\">");
            tableScript.Append("  小数");
            tableScript.Append(" </td>");
            tableScript.Append("<td class=\"td-00\">");
            tableScript.Append("   主键");
            tableScript.Append("</td>");
            tableScript.Append("<td class=\"td-00\">");
            tableScript.Append("     外键");
            tableScript.Append(" </td>");
            tableScript.Append(" <td class=\"td-00\">");
            tableScript.Append("标识");
            tableScript.Append(" </td>");
            tableScript.Append(" <td class=\"td-00\">");
            tableScript.Append("   自增长度");
            tableScript.Append("</td>");
            tableScript.Append("<td class=\"td-00\">");
            tableScript.Append("    空值");
            tableScript.Append(" </td>");
            tableScript.Append(" </tr>");
        }

        /// <summary>
        /// 表列、约束、触发和索引的缺损(跟表关联的)    
        /// </summary>
        public void PublicPartAboutTable(string ColumnOne, string ColumnTwo)
        {
            tableWrong = true;
            columnScript.Append("<tr>");
            columnScript.Append("<td class=\"td-04\">");
            columnScript.Append(ColumnOne);
            columnScript.Append("</td>");
            columnScript.Append("<td class=\"td-04\">");
            columnScript.Append(ColumnTwo);
            columnScript.Append("</td>");
            columnScript.Append("<td class=\"td-04\">");
            columnScript.Append("</td>");
            columnScript.Append("  <td class=\"td-04\">");
            columnScript.Append(" </td>");
            columnScript.Append("<td class=\"td-04\">");
            columnScript.Append("</td>");
            columnScript.Append("<td class=\"td-04\">");
            columnScript.Append(" </td>");
            columnScript.Append(" <td class=\"td-04\">");
            columnScript.Append(" </td>");
            columnScript.Append(" <td class=\"td-04\">");
            columnScript.Append("</td>");
            columnScript.Append("<td class=\"td-04\">");
            columnScript.Append(" </td>");
            columnScript.Append("<td class=\"td-04\">");
            columnScript.Append(" </td>");
            columnScript.Append("</tr>");
        }


        /// <summary>
        /// 表，视图和存储过程的缺损
        /// </summary>
        /// <param name="Reasones"></param>
        /// <param name="table"></param>
        /// <param name="viewbad"></param>
        /// <param name="viewlack"></param>
        /// <param name="proceduebad"></param>
        /// <param name="proceduelack"></param>
        public string PublicPartAboutDataBase(string[] Reasones, List<string> table, List<string> viewbad, List<string> viewlack, List<string> proceduebad, List<string> proceduelack)
        {
            bool[] notEmpty = new bool[5];
            databaseScript = new StringBuilder();
            databaseScript.Append("<table class=\"Table-00\" cellspacing=\"1\" cellpadding=\"0\">");
            databaseScript.Append("<tr>");
            for (int i = 0; i < 5; i++)
            {
                if (reason[i] != null)
                {
                    notEmpty[i] = true;
                    databaseScript.Append("<td class=\"td-00\" colspan=\"9\">");
                    databaseScript.Append(reason[i] + "</td>");
                    databaseScript.Append("<td class=\"td-00\" colspan=\"9\">");
                    databaseScript.Append("</td>");
                }
            }
            databaseScript.Append("</tr>");
            if (notEmpty != null)
            {
                int bigger = CompareBigger(table.Count, viewbad.Count, viewlack.Count, proceduebad.Count, proceduelack.Count);
                for (int j = 0; j < bigger; j++)
                {
                    databaseScript.Append("<tr>");
                    if (notEmpty[0])
                    {
                        if (j < table.Count)
                            TVPLackOrBad(table[j]);
                        else
                        {
                            databaseScript.Append("<td class=\"td-01\" colspan=\"9\">");
                            databaseScript.Append("</td>");
                            databaseScript.Append("<td class=\"td-01\" colspan=\"9\">");
                            databaseScript.Append("</td>");
                        }
                    }
                    if (notEmpty[1])
                    {
                        if (j < viewbad.Count)
                            TVPLackOrBad(viewbad[j]);
                        else
                        {
                            databaseScript.Append("<td class=\"td-01\" colspan=\"9\">");
                            databaseScript.Append("</td>");
                            databaseScript.Append("<td class=\"td-01\" colspan=\"9\">");
                            databaseScript.Append("</td>");
                        }
                    }
                    if (notEmpty[2])
                    {
                        if (j < viewlack.Count)
                            TVPLackOrBad(viewlack[j]);
                        else
                        {
                            databaseScript.Append("<td class=\"td-01\" colspan=\"9\">");
                            databaseScript.Append("</td>");
                            databaseScript.Append("<td class=\"td-01\" colspan=\"9\">");
                            databaseScript.Append("</td>");
                        }
                    }
                    if (notEmpty[3])
                    {
                        if (j < proceduebad.Count)
                            TVPLackOrBad(proceduebad[j]);
                        else
                        {
                            databaseScript.Append("<td class=\"td-01\" colspan=\"9\">");
                            databaseScript.Append("</td>");
                            databaseScript.Append("<td class=\"td-01\" colspan=\"9\">");
                            databaseScript.Append("</td>");
                        }
                    }
                    if (notEmpty[4])
                    {
                        if (j < proceduelack.Count)
                            TVPLackOrBad(proceduelack[j]);
                        else
                        {
                            databaseScript.Append("<td class=\"td-01\" colspan=\"9\">");
                            databaseScript.Append("</td>");
                            databaseScript.Append("<td class=\"td-01\" colspan=\"9\">");
                            databaseScript.Append("</td>");
                        }
                    }
                    databaseScript.Append("</tr>");
                }
                databaseScript.Append("</table>");
                return databaseScript.ToString();
            }
            else
                return string.Empty;

        }

        public void TVPLackOrBad(string name)
        {
            if (name != string.Empty)
            {
                databaseScript.Append("<td class=\"td-01\" colspan=\"9\">");
                databaseScript.Append(name + "</td>");
                databaseScript.Append("<td class=\"td-01\" colspan=\"9\">");
                databaseScript.Append("</td>");
            }
            else
            {
                databaseScript.Append("<td class=\"td-01\" colspan=\"9\">");
                databaseScript.Append("</td>");
                databaseScript.Append("<td class=\"td-01\" colspan=\"9\">");
                databaseScript.Append("</td>");
            }
        }

        public int CompareBigger(int t, int vb, int vl, int pb, int pl)
        {
            int tmp = t;
            if (tmp < vb)
                tmp = vb;
            if (tmp < vl)
                tmp = vl;
            if (tmp < pb)
                tmp = pb;
            if (tmp < pl)
                tmp = pl;
            return tmp;
        }

        public List<string>[] TableViewProcecure(string serverName, string userName, string password, string databaseName)
        {
            string connstr = "Data Source=" + serverName + ";Initial Catalog=" + databaseName + ";User ID=" + userName + ";pwd=" + password;
            List<string>[] TVP = new List<string>[3];
            SqlConnection conn = new SqlConnection(connstr);
            SqlCommand comm = new SqlCommand();
            comm.Connection = conn;
            comm.Connection.Open();

            #region 表名
            string sql = "select name from sysobjects where xtype='U'"; //表名
            comm.CommandText = sql;

            SqlDataReader dr = comm.ExecuteReader();
            if (dr != null)
            {
                TVP[0] = new List<string>();
                while (dr.Read())
                {

                    if (dr["name"] != DBNull.Value)
                    {
                        TVP[0].Add(Convert.ToString(dr["name"]));
                    }
                }
                dr.Close();
            }
            #endregion

            #region 视图名
            sql = "select name from sysobjects where xtype='V'"; //视图名
            comm.CommandText = sql;

            dr = comm.ExecuteReader();
            if (dr != null)
            {
                TVP[1] = new List<string>();
                while (dr.Read())
                {

                    if (dr["name"] != DBNull.Value)
                    {
                        TVP[1].Add(Convert.ToString(dr["name"]));
                    }
                }
                dr.Close();
            }
            #endregion

            #region 存储过程
            sql = "select name from sysobjects where xtype='P'"; //存储过程
            comm.CommandText = sql;

            dr = comm.ExecuteReader();
            if (dr != null)
            {
                TVP[2] = new List<string>();
                while (dr.Read())
                {

                    if (dr["name"] != DBNull.Value)
                    {
                        TVP[2].Add(Convert.ToString(dr["name"]));
                    }
                }
                dr.Close();
            }
            #endregion
            conn.Close();
            return TVP;
        }

       
    }
}
